@@ -33,9 +33,11 @@
 #include "signals.h"
 #include "levels.h"
 #include "settings.h"
+#include "ignore.h"
 #include "fe-common/core/printtext.h"
 #include "fe-common/core/fe-channels.h"
 #include "fe-common/core/keyboard.h"
+#include "fe-common/core/window-items.h"
 #include "fe-common/silc/module-formats.h"
 
 #include "core.h"
@@ -81,6 +83,47 @@ static void silc_get_umode_string(SilcUInt32 mode, char *buf,
     strcat(buf, " [blocks invites]");
 }
 
+/* print "nick appears as" message to every channel of a server */
+static void 
+silc_print_nick_change_channel(SILC_SERVER_REC *server, const char *channel,
+			      const char *newnick, const char *oldnick,
+			      const char *address)
+{
+  if (ignore_check(SERVER(server), oldnick, address,
+		   channel, newnick, MSGLEVEL_NICKS))
+    return;
+  
+  printformat_module("fe-common/silc", server, channel, MSGLEVEL_NICKS,
+		     SILCTXT_CHANNEL_APPEARS,
+		     oldnick, newnick, channel, address);
+}
+
+static void
+silc_print_nick_change(SILC_SERVER_REC *server, const char *newnick,
+		       const char *oldnick, const char *address)
+{
+  GSList *tmp, *windows;
+
+  /* Print to each channel/query where the nick is.
+     Don't print more than once to the same window. */
+  windows = NULL;
+    
+  for (tmp = server->channels; tmp != NULL; tmp = tmp->next) {
+    CHANNEL_REC *channel = tmp->data;
+    WINDOW_REC *window = window_item_window((WI_ITEM_REC *) channel);
+
+    if (nicklist_find(channel, newnick) == NULL ||
+	g_slist_find(windows, window) != NULL)
+      continue;
+
+    windows = g_slist_append(windows, window);
+    silc_print_nick_change_channel(server, channel->visible_name,
+				   newnick, oldnick, address);
+  }
+
+  g_slist_free(windows);
+}
+
 void silc_say(SilcClient client, SilcClientConnection conn,
 	      SilcClientMessageType type, char *msg, ...)
 {
@@ -434,9 +477,26 @@ void silc_notify(SilcClient client, SilcClientConnection conn,
     
     chanrec = silc_channel_find_entry(server, channel);
     if (chanrec != NULL) {
+      char tmp2[256], *cp, *dm = NULL;
+
       g_free_not_null(chanrec->topic);
+      if (tmp && !silc_term_utf8() && silc_utf8_valid(tmp, strlen(tmp))) {
+	memset(tmp2, 0, sizeof(tmp2));
+	cp = tmp2;
+	if (strlen(tmp) > sizeof(tmp2) - 1) {
+	  dm = silc_calloc(strlen(tmp) + 1, sizeof(*dm));
+	  cp = dm;
+	}
+
+	silc_utf8_decode(tmp, strlen(tmp), SILC_STRING_LANGUAGE,
+			 cp, strlen(tmp));
+	tmp = cp;
+      }
+
       chanrec->topic = *tmp == '\0' ? NULL : g_strdup(tmp);
       signal_emit("channel topic changed", 1, chanrec);
+
+      silc_free(dm);
     }
     
     if (idtype == SILC_ID_CLIENT) {
@@ -1042,6 +1102,118 @@ void silc_getkey_cb(bool success, void *context)
   silc_free(getkey);
 }
 
+/* Parse an invite or ban list */
+void  silc_parse_inviteban_list(SilcClient client,
+				SilcClientConnection conn,
+				SILC_SERVER_REC *server, 
+				SilcChannelEntry channel, 
+				const char *list_type,
+				SilcArgumentPayload list)
+{
+  unsigned char *tmp;
+  SilcUInt32 type, len;
+  SILC_CHANNEL_REC *chanrec = silc_channel_find_entry(server, channel);
+  int counter=0, resolving = FALSE;
+
+  if (!silc_argument_get_arg_num(list)) {
+    printformat_module("fe-common/silc", server,
+		       (chanrec ? chanrec->visible_name : NULL),
+		       MSGLEVEL_CRAP, SILCTXT_CHANNEL_NO_INVITEBAN_LIST,
+		       channel->channel_name, list_type);
+    return;
+  }
+  
+  printformat_module("fe-common/silc", server,
+		     (chanrec ? chanrec->visible_name : NULL),
+		     MSGLEVEL_CRAP, SILCTXT_CHANNEL_INVITEBAN_LIST,
+		     channel->channel_name, list_type);
+
+  /* parse the list */
+  tmp = silc_argument_get_first_arg(list, &type, &len);
+  while (tmp) {
+    switch (type) {
+      case 1:
+	{
+	  /* an invite string */
+	  char **list;
+	  int i=0;
+		
+	  if (tmp[len-1] == ',')
+	    tmp[len-1] = '\0';
+	  
+	  list = g_strsplit(tmp, ",", -1);
+	  while (list[i])
+	    printformat_module("fe-common/silc", server,
+			       (chanrec ? chanrec->visible_name : NULL),
+			       MSGLEVEL_CRAP, SILCTXT_CHANNEL_INVITEBAN_STRING,
+			       ++counter, channel->channel_name, list_type,
+			       list[i++]);
+	  g_strfreev(list);
+	}
+	break;
+
+      case 2:
+	{
+	  /* a public key */
+	  char *fingerprint, *babbleprint;
+
+	  fingerprint = silc_hash_fingerprint(NULL, tmp, len);
+	  babbleprint = silc_hash_fingerprint(NULL, tmp, len);
+
+	  printformat_module("fe-common/silc", server,
+			     (chanrec ? chanrec->visible_name : NULL),
+			     MSGLEVEL_CRAP, SILCTXT_CHANNEL_INVITEBAN_PUBKEY,
+			     ++counter, channel->channel_name, list_type,
+			     fingerprint, babbleprint);
+	}
+	break;
+	
+      case 3:
+	{
+	  /* a client ID */
+	  SilcClientID *client_id;
+	  SilcClientEntry client_entry;
+	  
+	  client_id = silc_id_payload_parse_id(tmp, len, NULL);
+			  
+	  if (client_id == NULL) {
+	    silc_say_error("Invalid data in %s list encountered", list_type);
+	    break;
+	  }
+
+	  client_entry = silc_client_get_client_by_id(client, conn, client_id);
+
+	  if (client_entry) {
+	    printformat_module("fe-common/silc", server,
+			       (chanrec ? chanrec->visible_name : NULL),
+			       MSGLEVEL_CRAP, SILCTXT_CHANNEL_INVITEBAN_STRING,
+			       ++counter, channel->channel_name, list_type,
+			       client_entry->nickname);
+	  } else {
+	    resolving = TRUE;
+	    silc_client_get_client_by_id_resolve(client, conn, client_id,
+						 NULL, NULL, NULL);
+	  }
+
+	  silc_free(client_id);
+	}
+	break;
+	
+      default:
+	/* "trash" */
+	silc_say_error("Unkown type in %s list: %u (len %u)",
+		       list_type, type, len);
+    }
+    tmp = silc_argument_get_next_arg(list, &type, &len);
+  }
+
+  if (resolving)
+    printformat_module("fe-common/silc", server, 
+		       (chanrec ? chanrec->visible_name : NULL),
+		       MSGLEVEL_CRAP, SILCTXT_CHANNEL_INVITEBAN_REGET,
+		       list_type, channel->channel_name);
+}
+
 /* Command reply handler. This function is called always in the command reply
    function. If error occurs it will be called as well. Normal scenario
    is that it will be called after the received command data has been parsed
@@ -1266,28 +1438,26 @@ silc_command_reply(SilcClient client, SilcClientConnection conn,
   case SILC_COMMAND_INVITE:
     {
       SilcChannelEntry channel;
-      char *invite_list;
-      SilcArgumentPayload args;
-      int argc = 0;
+      SilcBuffer payload;
+      SilcArgumentPayload invite_list;
+      SilcUInt32 argc;
       
       if (!success)
 	return;
       
       channel = va_arg(vp, SilcChannelEntry);
-      invite_list = va_arg(vp, char *);
+      payload = va_arg(vp, SilcBuffer);
 
-      args = silc_command_get_args(cmd_payload);
-      if (args)
-	argc = silc_argument_get_arg_num(args);
-
-      if (invite_list)
-	printformat_module("fe-common/silc", server, NULL, MSGLEVEL_CRAP,
-			   SILCTXT_CHANNEL_INVITE_LIST, channel->channel_name,
-			   invite_list);
-      else if (argc == 3)
-	printformat_module("fe-common/silc", server, NULL, MSGLEVEL_CRAP,
-			   SILCTXT_CHANNEL_NO_INVITE_LIST, 
-			   channel->channel_name);
+      if (payload) {
+	SILC_GET16_MSB(argc, payload->data);
+	invite_list = silc_argument_payload_parse(payload->data + 2, 
+						  payload->len - 2, argc);
+	if (invite_list) {
+	  silc_parse_inviteban_list(client, conn, server, channel, 
+				    "invite", invite_list);
+	  silc_argument_payload_free(invite_list);
+	}
+      }
     }
     break;
 
@@ -1319,9 +1489,26 @@ silc_command_reply(SilcClient client, SilcClientConnection conn,
 	chanrec = silc_channel_create(server, channel, channel, TRUE);
 
       if (topic) {
+	char tmp[256], *cp, *dm = NULL;
 	g_free_not_null(chanrec->topic);
+
+	if (!silc_term_utf8() && silc_utf8_valid(topic, strlen(topic))) {
+	  memset(tmp, 0, sizeof(tmp));
+	  cp = tmp;
+	  if (strlen(topic) > sizeof(tmp) - 1) {
+	    dm = silc_calloc(strlen(topic) + 1, sizeof(*dm));
+	    cp = dm;
+	  }
+
+	  silc_utf8_decode(topic, strlen(topic), SILC_STRING_LANGUAGE,
+			   cp, strlen(topic));
+	  topic = cp;
+	}
+
 	chanrec->topic = *topic == '\0' ? NULL : g_strdup(topic);
 	signal_emit("channel topic changed", 1, chanrec);
+
+	silc_free(dm);
       }
 
       mode = silc_client_chmode(modei, 
@@ -1345,16 +1532,37 @@ silc_command_reply(SilcClient client, SilcClientConnection conn,
   case SILC_COMMAND_NICK: 
     {
       char *old;
-      SilcClientEntry client = va_arg(vp, SilcClientEntry);
+      SilcClientEntry client_entry = va_arg(vp, SilcClientEntry);
+      GSList *nicks;
       
       if (!success)
 	return;
 
+      nicks = nicklist_get_same(SERVER(server), client_entry->nickname);
+      if (nicks != NULL) {
+	char buf[512];
+	SilcClientEntry collider, old;
+
+	old = ((SILC_NICK_REC *)(nicks->next->data))->silc_user->client;
+	collider = silc_client_get_client_by_id(client, conn,
+						old->id);
+	
+        memset(buf, 0, sizeof(buf));
+        snprintf(buf, sizeof(buf) - 1, "%s@%s",
+		 collider->username, collider->hostname);
+	nicklist_rename_unique(SERVER(server),
+			       old, old->nickname,
+			       collider, collider->nickname);
+	silc_print_nick_change(server, collider->nickname,
+			       client_entry->nickname, buf);
+	g_slist_free(nicks);
+      }
+
       old = g_strdup(server->nick);
-      server_change_nick(SERVER(server), client->nickname);
+      server_change_nick(SERVER(server), client_entry->nickname);
       nicklist_rename_unique(SERVER(server),
 			     server->conn->local_entry, server->nick,
-			     client, client->nickname);
+			     client_entry, client_entry->nickname);
       signal_emit("message own_nick", 4, server, server->nick, old, "");
       g_free(old);
       break;
@@ -1408,6 +1616,7 @@ silc_command_reply(SilcClient client, SilcClientConnection conn,
   case SILC_COMMAND_UMODE:
     {
       SilcUInt32 mode;
+      char *reason;
       
       if (!success)
 	return;
@@ -1424,6 +1633,20 @@ silc_command_reply(SilcClient client, SilcClientConnection conn,
 	printformat_module("fe-common/silc", server, NULL,
 			   MSGLEVEL_CRAP, SILCTXT_ROUTER_OPER);
 
+      if ((mode & SILC_UMODE_GONE) != (server->umode & SILC_UMODE_GONE)) {
+	if (mode & SILC_UMODE_GONE) {      
+	  if ((server->away_reason != NULL) && (server->away_reason[0] != '\0'))
+	    reason = g_strdup(server->away_reason);
+	  else
+	    reason = g_strdup("away");
+	} else
+	  reason = g_strdup("");
+
+	silc_set_away(reason, server);
+
+	g_free(reason);
+      }
+
       server->umode = mode;
       signal_emit("user mode changed", 2, server, NULL);
     }
@@ -1511,22 +1734,26 @@ silc_command_reply(SilcClient client, SilcClientConnection conn,
   case SILC_COMMAND_BAN:
     {
       SilcChannelEntry channel;
-      char *ban_list;
+      SilcBuffer payload;
+      SilcArgumentPayload ban_list;
+      SilcUInt32 argc;
       
       if (!success)
 	return;
       
       channel = va_arg(vp, SilcChannelEntry);
-      ban_list = va_arg(vp, char *);
-      
-      if (ban_list)
-	printformat_module("fe-common/silc", server, NULL, MSGLEVEL_CRAP,
-			   SILCTXT_CHANNEL_BAN_LIST, channel->channel_name,
-			   ban_list);
-      else
-	printformat_module("fe-common/silc", server, NULL, MSGLEVEL_CRAP,
-			   SILCTXT_CHANNEL_NO_BAN_LIST, 
-			   channel->channel_name);
+      payload = va_arg(vp, SilcBuffer);
+
+      if (payload) {
+	SILC_GET16_MSB(argc, payload->data);
+	ban_list = silc_argument_payload_parse(payload->data + 2, 
+					       payload->len - 2, argc);
+	if (ban_list) {
+	  silc_parse_inviteban_list(client, conn, server, channel, 
+				    "ban", ban_list);
+	  silc_argument_payload_free(ban_list);
+	}
+      }
     }
     break;
     
@@ -1912,8 +2139,7 @@ silc_verify_public_key_internal(SilcClient client, SilcClientConnection conn,
   verify->entity_name = (conn_type != SILC_SOCKET_TYPE_CLIENT ?
 			 (name ? strdup(name) : strdup(conn->sock->hostname))
 			 : NULL);
-  verify->pk = silc_calloc(pk_len, sizeof(*verify->pk));
-  memcpy(verify->pk, pk, pk_len);
+  verify->pk = silc_memdup(pk, pk_len);
   verify->pk_len = pk_len;
   verify->pk_type = pk_type;
   verify->completion = completion;
@@ -2021,6 +2247,11 @@ silc_verify_public_key_internal(SilcClient client, SilcClientConnection conn,
     if (completion)
       completion(TRUE, context);
     silc_free(fingerprint);
+    silc_free(verify->filename);
+    silc_free(verify->entity);
+    silc_free(verify->entity_name);
+    silc_free(verify->pk);
+    silc_free(verify);
   }
 }
 
