@@ -17,7 +17,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: silccipher.c,v 1.22 2002/10/24 12:18:59 priikone Exp $ */
+/* $Id: silccipher.c,v 1.24 2002/11/24 16:28:07 priikone Exp $ */
 
 #include "silcincludes.h"
 #include "ciphers.h"		/* Includes cipher definitions */
@@ -67,15 +67,6 @@ const SilcCipherObject silc_default_ciphers[] =
   { "rc6-128-cbc", 16, 128, silc_rc6_set_key, silc_rc6_set_key_with_string,
     silc_rc6_encrypt_cbc, silc_rc6_decrypt_cbc, 
     silc_rc6_context_len },
-  { "mars-256-cbc", 16, 256, silc_mars_set_key, silc_mars_set_key_with_string,
-    silc_mars_encrypt_cbc, silc_mars_decrypt_cbc, 
-    silc_mars_context_len },
-  { "mars-192-cbc", 16, 192, silc_mars_set_key, silc_mars_set_key_with_string,
-    silc_mars_encrypt_cbc, silc_mars_decrypt_cbc, 
-    silc_mars_context_len },
-  { "mars-128-cbc", 16, 128, silc_mars_set_key, silc_mars_set_key_with_string,
-    silc_mars_encrypt_cbc, silc_mars_decrypt_cbc, 
-    silc_mars_context_len },
   { "cast-256-cbc", 16, 256, silc_cast_set_key, silc_cast_set_key_with_string,
     silc_cast_encrypt_cbc, silc_cast_decrypt_cbc, 
     silc_cast_context_len },
@@ -324,7 +315,8 @@ bool silc_cipher_encrypt(SilcCipher cipher, const unsigned char *src,
 			 unsigned char *dst, SilcUInt32 len, 
 			 unsigned char *iv)
 {
-  return cipher->cipher->encrypt(cipher->context, src, dst, len, iv);
+  return cipher->cipher->encrypt(cipher->context, src, dst, len,
+				 iv ? iv : cipher->iv);
 }
 
 /* Decrypts */
@@ -333,7 +325,8 @@ bool silc_cipher_decrypt(SilcCipher cipher, const unsigned char *src,
 			 unsigned char *dst, SilcUInt32 len, 
 			 unsigned char *iv)
 {
-  return cipher->cipher->decrypt(cipher->context, src, dst, len, iv);
+  return cipher->cipher->decrypt(cipher->context, src, dst, len,
+				 iv ? iv : cipher->iv);
 }
 
 /* Sets the key for the cipher */
